package org.mdp.cli;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.rmi.AccessException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Map;

import org.mdp.dir.User;
import org.mdp.dir.UserDirectoryStub;


public class InstantMessagingApp {

	static String LOGO = 
			"========================================\n"+
					" _ __ ___   ___ _ __  ___  __ _ (_) ___\n"+ 
					"| '_ ` _ \\ / _ \\ '_ \\/ __|/ _` || |/ _ \n"+
					"| | | | | |  __/ | | \\__ \\ (_| || |  __/\n"+
					"|_| |_| |_|\\___|_| |_|___/\\__,_|/ |\\___|\n"+
					"                               |__/     \n"+
					"========================================\n";

	static String INTRO = LOGO +"... a revolution in communication!\n";


	/** 
	 * @param args
	 * @throws IOException
	 * @throws AlreadyBoundException
	 * @throws NotBoundException 
	 */
	public static void main(String args[]) throws IOException, AlreadyBoundException, NotBoundException{
		System.out.println(INTRO);

		// Create a user for yourself
		// Please don't call your username: 'list' 'rmv' or 'add' :P
		// If you like, pass info through the command-line instead
		// of hard-coding (optional)
		User me = new User("test", "Aidan Hogan", "localhost", 1915);

		// Then start your RMI registry ...
		Registry reg = startRegistry(me.getPort());
		
		// and your message server to start receiving messages ...
		// need to keep the reference to server even if not used!
		@SuppressWarnings("unused")
		Remote server = bindSkeleton(reg);

		// Connect to the directory
		UserDirectoryStub uds = connectToDirectory();

		Map<String,User> dir = uds.getDirectory();

		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		
		while(true){
			System.err.println("\n\nType 'list' to up-to-date directory, 'add' to add yourself to the directory, 'rmv' to remove yourself from directory, or 'msg' to send a message\n");
			
			String line = br.readLine().trim();
			
			if(line.equals("list")){
				dir = uds.getDirectory();
				System.err.println("\n\nDirectory of usernames ...");
				for(String s: dir.keySet()){
					System.err.println(s+" ");
				}
			} else if(line.equals("rmv")){
				uds.removeUserWithName(me.getUsername());
			} else if(line.equals("add")){
				uds.createUser(me);
			} else if(line.equals("msg")){
				System.out.println("\n\nWhich user would you like to message (or type 'all') ...");
				
				line = br.readLine().trim();
				
				String target = line;
				ArrayList<String> usernames = new ArrayList<String>();
				
				if(target.equals("all")){
					usernames.addAll(dir.keySet());
				} else{
					usernames.add(target);
				}
				
				for(String username:usernames){
				
					User user = dir.get(username);
					if(user==null){
						System.out.println("\n\nCould not find the user "+username+" in the directory. (Type 'list' to refresh directory.)");
					} else{
						System.out.println("\n\nEnter your message ...");
						
						line = br.readLine().trim();
		
						try{
							long ms = messageUser(user,me,line);
							System.out.println("\n\nMessage received by "+username+" at time "+ms);
						} catch(Exception e){
							e.printStackTrace();
							System.out.println("\n\nCould not message user "+username+": "+e.getMessage());
						}
					}
				}
			} else{
				System.out.println("Unrecognised command "+line);
			}
		}

	}

	/**
	 * Starts your registry on the given port.
	 * 
	 * @param port
	 * @return
	 * @throws RemoteException
	 * @throws AlreadyBoundException
	 */
	public static Registry startRegistry(int port) throws RemoteException, AlreadyBoundException {
		// open and return the registry on the port
		// ...
		
		return null; // <- replace
	}
	
	/**
	 * Create a skeleton and bind it to the registry
	 * 
	 * @param r
	 * @return
	 * @throws RemoteException
	 * @throws AlreadyBoundException
	 */
	public static Remote bindSkeleton(Registry r) throws RemoteException, AlreadyBoundException {
		// create a skeleton/stub for the instant messaging server ...
		// ...
		
		// exportObject just creates a stub that can be called
		// remotely by the client...
		// ...
		

		// pick the class name for the stub (any name would be fine
		// but client needs to know the name to find the stub)
		// ...
		
		
		// bind the stub to the registry under the given name
		// ...

		// return the skeleton
		return null; //<- replace
	}

	/**
	 * Message user.
	 * 
	 * Connect to their registry, retrieve the InstantMessagingStub stub.
	 * Call the message method on the stub.
	 * 
	 * @param username
	 * @param msg
	 * @return Time acknowledged by message server
	 * @throws NotBoundException 
	 * @throws RemoteException 
	 * @throws AccessException 
	 */
	public static long messageUser(User to, User from, String msg) throws AccessException, RemoteException, NotBoundException{
		// first need to connect to the remote registry on the given
		// IP and port for the user "to" to send a message
		// ...

		// then need to find the interface we're looking for (and cast it)
		// ...
		
		
		// and message user
		// ...
		
		return 0l; // and return time given by messaging user 
	}
	
	/**
	 * Connect to the central directory.
	 * 
	 * (See UserDirectoryClient for example)
	 * 
	 * 
	 * @return A stub to call message method on
	 * @throws RemoteException 
	 * @throws NotBoundException 
	 */
	public static UserDirectoryStub connectToDirectory() throws RemoteException, NotBoundException {
		// first need to connect to the remote registry on the given
		// IP and port to connect to directory
		// ...

		// then need to find the interface we're looking for (and cast it) 
		// ...
		
		// and return it
		return null; //<- replace
	}
}
